/**
 * Example:
 * {
 * "paths": {
 *  "9090":"HXW3fnKwUULrfKS1-cdb",
 *  "3000":"HXW3fnKwUULrfKS1-web",
 *  "8080":"HXW3fnKwUULrfKS1-angular"
 *  },
 * "host":"dev.deadlock.io",
 * "token":"e75da487-9576-4427-be2d-8da4059adc7d",
 * "giteaHost":"deadlock.minikube",
 * "giteaSshPort":32000,
 * "username":"slayug",
 * "missionId":"code_persist_cdb_crud"
 * }
 */
import { recorderError as error } from '../recorder/utils/log';
import { UserDetails } from './mission/models/userChallenge';

export default abstract class UserConfig {
  private userConfigJson: any | undefined;

  getPaths(): Map<number, string> {
    return this.userConfigJson?.paths;
  }

  getHost(): string {
    return this.userConfigJson?.host;
  }
  getGiteaHost(): string {
    return this.userConfigJson?.giteaHost;
  }

  getGiteaSshPort(): number {
    return this.userConfigJson?.giteaSshPort;
  }

  getUsername(): string {
    return this.userConfigJson?.username;
  }

  getRemoteGitUsername(): string {
    return this.userConfigJson?.remoteGitUsername;
  }

  getMissionId(): string {
    return this.userConfigJson?.missionId;
  }

  getEmail(): string {
    return this.userConfigJson?.email;
  }

  isProfessor(): boolean {
    return this.userConfigJson?.professor;
  }

  getCurrentUserDetails(): UserDetails {
    return this.userConfigJson?.currentUserDetails;
  }

  getRemoteUserDetails(): UserDetails {
    return this.userConfigJson?.remoteUserDetails;
  }

  getCurrentUserId(): string {
    return this.userConfigJson?.currentUserId;
  }

  abstract loadText(): Promise<string>;

  public async init() {
    try {
      const userConfig = await this.loadText();

      this.userConfigJson = JSON.parse(userConfig);
      return Promise.resolve();
    } catch (e) {
      error('Cannot load userConfig');
      error(e);
      return Promise.reject();
    }
  }
}
