import { window, ExtensionContext, workspace, commands } from 'vscode';
import Controller from './core/controller';
import isDocker from './core/utils/isdocker';
import Recorder from './recorder/recorder';
import { extensionError as error } from './recorder/utils/log';
import { DepNodeProvider } from './theia/deadlockPanel';
import UserConfigTheia from './theia/userConfigTheia';
import { CommandTreeProvider } from './view/CommandTree';
import StartedMissionsView from './view/startedMissionsView';

// TODO: refactor remove this
export const userConfig = new UserConfigTheia();

export async function runTimer(userId: string, missionId: string, context: ExtensionContext) {
  const apiService = Controller.getInstance(context).apiService;
  try {
    await apiService.startUpdateWorkTime(userId, missionId);
    setInterval(async () => {
      await apiService.pingUpdateWorkTime(userId, missionId);
    }, 60000); // 1 minute
  } catch (e) {
    window.showErrorMessage(`Erreur lors de l'enregistrement du temps de travail`);
    error(JSON.stringify(e));
  }
}

export async function activate(context: ExtensionContext) {
  window.showInformationMessage('Bienvenue sur Deadlock!');
  Controller.getInstance(context);

  const workspaceFolders = workspace.workspaceFolders?.toString() ?? '';
  if (!workspaceFolders) window.showInformationMessage('Pas de répertoires ouverts');
  const deadlockPanelProvider = new DepNodeProvider();
  window.registerTreeDataProvider('deadlockPanel', deadlockPanelProvider);
  window.registerWebviewViewProvider('startedMissions', new StartedMissionsView(context));

  if (isDocker()) {
    commands.executeCommand('setContext', 'deadlock.inContainer', true);
    window.registerTreeDataProvider('commandTree', new CommandTreeProvider(context));
  } else {
    commands.executeCommand('setContext', 'deadlock.inContainer', false);
  }
  try {
    await userConfig.init();
  } catch (e) {
    error(JSON.stringify(e));
    error('Cannot init userConfig');
  }
  if (isDocker()) {
    try {
      Recorder.instance.run();
    } catch (e) {
      error('Could not run recorder');
      error(JSON.stringify(e));
      window.showErrorMessage("Le recorder n'a pas pu être lancé");
    }
    runTimer(userConfig.getCurrentUserId(), userConfig.getMissionId(), context);
  }
}
