import { Branch } from '../core/gitMission';
import { ENABLE_AUTOMATIC_SAVE } from '../config';
import GitMission from '../core/gitMission';
import UserConfigNode from './userConfigNode';
import { PROJECT_DEADLOCK_DESKTOP_PATH, PROJECT_SRC_PATH } from '../core/config';
import { recorderError as error } from './utils/log';
import UserConfig from '../core/userConfig';
import AutomaticSave from './services/automatic-save';
import { clearFilesExceptGit, copyProjectSources, renameTempToUserGitFiles } from './utils/workdir';
import aquirePermissions from './utils/permission';

export default class Recorder {
  private _gitMission?: GitMission;
  private userConfig?: UserConfigNode;
  private static _instance?: Recorder;

  private constructor() {
    Recorder._instance = this;
  }

  public static get instance(): Recorder {
    if (!this._instance) {
      return new Recorder();
    }
    return this._instance;
  }

  public get gitMission(): GitMission {
    // TODO refactor remove !
    return this._gitMission!;
  }

  async setupProject(userConfig: UserConfig, gitMission?: GitMission) {
    if (!userConfig.isProfessor()) {
      await copyProjectSources(PROJECT_SRC_PATH, PROJECT_DEADLOCK_DESKTOP_PATH, ['.git/']);

      if (gitMission) {
        renameTempToUserGitFiles(PROJECT_DEADLOCK_DESKTOP_PATH, gitMission.author);
      } else {
        error('Cannot start command recorder, gitMission not found');
      }
    } else {
      await copyProjectSources(PROJECT_SRC_PATH, PROJECT_DEADLOCK_DESKTOP_PATH);
    }
  }

  async setupFromRemoteRepo(gitMission: GitMission) {
    const isRemoteRepoExist = await gitMission.isRemoteRepoExist();
    if (isRemoteRepoExist) {
      // rm all except git directory pull remote code and setup
      await clearFilesExceptGit(PROJECT_SRC_PATH);

      await gitMission.fetch();
      await gitMission.checkout(Branch.MASTER);
      await gitMission.checkout(Branch.LIVE);
    } else {
      await gitMission.commit('initial commit', ['--allow-empty']);
      await gitMission.createRemoteBranch(Branch.DEFAULT);
      await gitMission.createBranch(Branch.LIVE);
    }
  }

  async run() {
    this.userConfig = new UserConfigNode();

    try {
      await aquirePermissions();
      // TODO refactor remove this
      await this.userConfig.init();
      this._gitMission = await new GitMission(this.userConfig).init();
      await this.setupFromRemoteRepo(this._gitMission);
      await this.setupProject(this.userConfig, this._gitMission);
      try {
        if (ENABLE_AUTOMATIC_SAVE) new AutomaticSave(PROJECT_DEADLOCK_DESKTOP_PATH, this._gitMission);
      } catch (e) {
        error('Error while setup automatic save');
        error(e);
      }
    } catch (e) {
      error('Cannot setup user repo.');
      error(e);
    }
  }
}
