import isDocker from '../core/utils/isdocker';
import ChallengeYaml, { MissionCommand as MissionCommandYaml } from '../model/challengeYaml';
import { parse } from 'yaml';
import { readFileSync } from 'fs';
import { commands, ExtensionContext, ThemeColor, ThemeIcon, TreeDataProvider, TreeItem, window } from 'vscode';
import { commitAndPushOnQueue, CommitFrom } from '../recorder/utils/gitea';
import Recorder from '../recorder/recorder';

export class CommandTreeProvider implements TreeDataProvider<MissionCommand> {
  challenge: ChallengeYaml;
  context: ExtensionContext;
  constructor(context: ExtensionContext) {
    this.context = context;
    if (isDocker()) {
      this.challenge = parse(readFileSync('/deadlock/challenge.yaml', 'utf8'));
    } else {
      window.showErrorMessage('No mission found');
      throw new Error('No mission found');
    }
  }

  getTreeItem(element: MissionCommand): TreeItem {
    return element;
  }

  getChildren(element?: MissionCommand): Thenable<MissionCommand[]> {
    if (element) {
      return Promise.resolve([]);
    } else {
      return Promise.resolve(this.challenge.desktop?.scripts?.map((script) => new MissionCommand(script)) ?? []);
    }
  }
}

class MissionCommand extends TreeItem {
  missionCommandYaml: MissionCommandYaml;
  constructor(missionCommandYaml: MissionCommandYaml) {
    super(missionCommandYaml.name);
    this.missionCommandYaml = missionCommandYaml;
    this.tooltip = missionCommandYaml.description;
    this.description = missionCommandYaml.description;
    this.iconPath = new ThemeIcon('notebook-execute', new ThemeColor('debugIcon.startForeground'));
    commands.registerCommand(missionCommandYaml.name, () => {
      const terminalName = `Deadlock`;
      const terminal =
        window.terminals.filter((terminal) => terminal.name === terminalName)[0] ??
        window.createTerminal({
          cwd: process.env.WORKDIR,
          name: terminalName,
        });
      terminal.sendText(missionCommandYaml.command);
      terminal.show();
      commitAndPushOnQueue(Recorder.instance.gitMission, CommitFrom.Run);
    });
    this.command = {
      title: missionCommandYaml.name,
      command: missionCommandYaml.name,
    };
  }
}
